<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Jan 19, 2021
 */

use IWorx\Process\System;

/**
 * Imunifyav plugin.
 *
 * Enables Imunifyav integration.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_Imunifyav extends Plugin
{
    const AV = 'av';
    const IM360  = 'im360';

    /**
     * init
     */
    protected function _init()
    {
    }

    /**
     * Pre action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function preAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        $version = Ini::get(Ini::IMUNIFY, 'application');
        if ($version === Plugin_Imunifyav::IM360) {
            if ($ctrl_act === 'Ctrl_Nodeworx_Firewall:index') {
                throw new IWorx_Exception_ActionBlocked('Imunify Firewall has replaced this item');
            } elseif (strpos($ctrl_act, 'Ctrl_Nodeworx_Firewall') === 0) {
                throw new IWorx_Exception_ActionBlocked('N/A');
            }
        }
    }

    /**
     * Post action.
     *
     * @param string        $ctrl_act Controller_Name:action
     * @param Ctrl_Abstract $Ctrl
     * @param string        $action
     * @param mixed         $params
     */
    public function postAction($ctrl_act, Ctrl_Abstract $Ctrl, $action, $params)
    {
        if ($ctrl_act === 'Ctrl_Siteworx_Ssl:installCommit') {
            if ($Ctrl->isViewSuccess()) {
                if ($this->is360()) {
                    //clean up cache first
                    $domain = $params->getValue('domain');
                    $Cmd    = new Command();
                    $Cmd->runAsRoot();
                    $Cmd->setCmd("im360-ssl-cache --remove {$domain}");
                    $Cmd->exec();

                    //add to cache
                    $SW     = new SiteWorx($domain);
                    $Driver = $SW->getHTTPDriver()->getSSLDriver();
                    $Driver->read();
                    $crt   = $Driver->getCRT();
                    $key   = $Driver->getKey();
                    $chain = $Driver->getChain();
                    $file  = "{$Driver->getSslDir()}/imunify.json";

                    $data = [
                      'domain'      => $domain,
                      'key'         => $key,
                      'certificate' => $crt
                    ];

                    if ($chain) {
                        $data['chain'] = $chain;
                    }

                    //im360-ssl-cache really wants those brackets.
                    $data = "[\n" . json_encode($data, JSON_PRETTY_PRINT) . "\n]";

                    IWorxFileSys::smartBarf($file, $data);
                    $Cmd = new Command();
                    $Cmd->runAsRoot();
                    $Cmd->setCmd("im360-ssl-cache --add {$file}");
                    $Cmd->exec();
                }
            }
        }

        if ($ctrl_act === 'Ctrl_Siteworx_Ssl:uninstallCommit') {
            if ($Ctrl->isViewSuccess()) {
                if ($this->is360()) {
                    $domain = $params->getValue('domain');
                    $Cmd    = new Command();
                    $Cmd->runAsRoot();
                    $Cmd->setCmd("/usr/sbin/im360-ssl-cache --remove {$domain}");
                    $Cmd->exec();
                }
            }
        }
    }

    /**
     * Determins if the imunify version is 360
     *
     * @return boolean
     */
    public function is360()
    {
        return Ini::get(Ini::IMUNIFY, 'application') === Plugin_Imunifyav::IM360;
    }

    /**
     * Get domains.
     *
     * @param object $SW
     * @return array
     */
    public function getDomains($SW = null)
    {
        $data = [];
        $VhostReader = new ApacheVhostReader();
        if ($SW) {
            $master_domains = $SW->getDomainNameList();
        } else {
            $master_domains = $VhostReader->getSiteWorxDomainList();
        }

        foreach ($master_domains as $master_domain) {
            $SW      = new SiteWorx($master_domain);
            $domains = $SW->getDomainNameList();
            foreach ($domains as $domain) {
                $is_master = ($domain === $SW->getMasterDomainName() ? true : false);

                $data['data'][$domain] = [
                  'owner'         => $SW->getUniqname(),
                  'document_root' => $SW->getDomainRoot($domain),
                  'is_main'       => $is_master
                ];
            }
        }
        $data['metadata'] = ['result' => 'ok'];
        return $data;
    }

    /**
     * Get Admins.
     * @return array
     */
    public function getAdmins()
    {
        $data = [];
        $NW = new NodeWorx(Nodeworx::MASTER_ID);
        $master_user = $NW->getUserList('master')[0]->toAssocArray();

        $data['data'][] = ['name' => $master_user['email'],
                           'unix_user' => 'root',
                           'locale_code' => 'EN_us',
                           'email' => $master_user['email'],
                           'is_main' => $master_user['userid'] == 1];

        $data['metadata'] = ['result' => 'ok'];
        if (is_null($data['data'])) {
            $data['data'] = [];
        }
        return $data;
    }

    /**
     * Get Modsec.
     * @param string $tmpfile
     * @return integer
     */
    public function getModsec($tmpfile) : int
    {
        $contents = file_get_contents($tmpfile);
        $lines = explode(PHP_EOL, $contents);
        if ($this->_validateInput($lines)) {
            $this->_writeConfigs($lines);
            if (!Apache::httpdCheckSyntaxStatic()) {
                $this->_revertConfigs($lines);
                $this->_restartApache();
                return CLI::NOT_OK;
            }
        }
        $this->_restartApache();
        return CLI::OK;
    }

    /**
     * Check that the input we receive from Imunify is valid
     *
     * @param array $lines
     * @return boolean
     */
    private function _validateInput(array $lines) : bool
    {
        foreach ($lines as $line) {
            if ($line !== '') {
                $config = json_decode($line, null, 512, JSON_OBJECT_AS_ARRAY);
                //basic test to make sure json is valid
                if (is_null($config)) {
                    IW::getLog()->error("Unable to decode JsonLine from Imunify360. Line: $line");
                    return false;
                }

                //make sure we have what we need
                if (!array_key_exists('domain', $config) ||
                    !array_key_exists('content', $config)) {
                    IW::getLog()->error("Imunify360 input does not contain required information. Line: $line");
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Write the domain-specific modsec configuration
     *
     * @param array $lines
     */
    private function _writeConfigs(array $lines)
    {
        $closure = function ($modsec_conf, $modsec_bak, $config) {
            if (IWorxFileSys::isFile($modsec_conf)) {
                $Cp = System\Cp::factory();
                $Cp->setSource($modsec_conf)
                    ->setDestination($modsec_bak)
                    ->setForce(true)
                    ->setPreservePerms(true)
                    ->runAsCurrentUser()
                    ->exec();
            }
            IWorxFileSys::barf($modsec_conf, $config['content']);
            $Chmod = System\Chmod::factory();
            $Chmod->setTarget($modsec_conf)
                ->setOctalMode(0644)
                ->runAsCurrentUser()
                ->exec();
        };
        $this->_forEachConfig($closure, $lines);
    }

    /**
     * Revert any domain-specific modsec configuration
     *
     * @param array $lines
     */
    private function _revertConfigs(array $lines)
    {
        IW::getLog()->error('Apache syntax check has failed while adding Imunify360 changes.');
        $closure = function ($modsec_conf, $modsec_bak, $config) {
            if (IWorxFileSys::isFile($modsec_bak)) {
                $Cp = System\Cp::factory();
                $Cp->setSource($modsec_bak)
                    ->setDestination($modsec_conf)
                    ->setForce(true)
                    ->setPreservePerms(true)
                    ->runAsCurrentUser()
                    ->exec();
            } else {
                $Rm = System\Rm::factory();
                $Rm->setTarget($modsec_conf)
                    ->runAsCurrentUser()
                    ->exec();
            }
        };
        $this->_forEachConfig($closure, $lines);
    }

    /**
     * Perform an action for each line of the config from Imunify
     *
     * @param closure $closure
     * @param array   $lines
     */
    private function _forEachConfig(closure $closure, array $lines)
    {
        foreach ($lines as $line) {
            if ($line !== '') {
                $config = json_decode($line, null, 512, JSON_OBJECT_AS_ARRAY);
                $Domain = new Domain($config['domain']);
                $apache_dir = $Domain->getLocalStateDir('apache');

                $modsec_conf = "$apache_dir/{$Domain->getName()}_modsec.conf";
                $modsec_bak = "$modsec_conf.bak";

                $closure($modsec_conf, $modsec_bak, $config);
            }
        }
    }

    /**
     *  Restarts Apache
     */
    private function _restartApache()
    {
        $httpd = new Apache();
        $httpd->restart();
    }


    /**
     * Get Users.
     *
     * @return array
     */
    public function getUsers()
    {
        $users = ShellAccount::getSiteworxUsers();
        $data  = [];

        foreach ($users as $user) {
            $uid = $user['uid'];
            $username = $user['user'];
            $SW = new SiteWorx($user['siteworx']);
            $email = $SW->getMasterUser()->getEmail();
            $domain = $SW->getMasterDomainName();
            $locale_code = 'EN_us';
            $owner = $SW->getParentMasterUser()->getEmail();
            $package_name = $SW->findPackageTemplateName();

            $data['data'][] = ['id' => intval($uid),
                               'username' => $username,
                               'owner' => $owner,
                               'domain' => $domain,
                               'package' => [
                                 'name' => $package_name,
                                 'owner' => $owner
                               ],
                               'email' => $email,
                               'locale_code' => $locale_code
            ];
        }

        if (is_null($data['data'])) {
            $data['data'] = [];
        }

        $data['metadata'] = ['result' => 'ok'];

        if (is_null($data['data'])) {
            $data['data'] = [];
        }

        return $data;
    }

    /**
     * Executes the open interface.
     *
     */
    public function runOpen()
    {
        include(Ini::get(Ini::IWORX_DIR, 'base') .'/plugins/imunifyav/lib/index.php');
    }

    /**
     * Executes the assets interface.
     *
     * @param string $filename
     * @param string $subdirectory
     */
    public function runAssets($filename, $subdirectory)
    {
        if (str_ends_with($filename, '.js')){
            header('Content-Type: text/javascript; charset=utf-8');
        }
        if (str_ends_with($filename, '.css')){
            header('Content-Type: text/css; charset=utf-8');
        }

        if (str_ends_with($filename, '.js.map')) {
            header('Content-Type: text/json; charset=utf-8');
        }
        if ($subdirectory === 'images') {
            include(__DIR__ . "/../lib/assets/static/nav-root/images/{$filename}");
        } elseif ($subdirectory === 'builds') {
            include(__DIR__ . "/../lib/assets/static/builds/{$filename}");
        }  elseif ($subdirectory === 'other-root') {
            include(__DIR__ . "/../lib/assets/static/other-root/{$filename}");
        }  elseif ($subdirectory === 'nav-root') {
            include(__DIR__ . "/../lib/assets/static/nav-root/{$filename}");
        }  elseif ($subdirectory === 'email-root') {
            include(__DIR__ . "/../lib/assets/static/email-root/{$filename}");
        } elseif ($subdirectory === 'config') {
            include(__DIR__ . "/../lib/assets/js/config.js");
        } elseif ($subdirectory === 'flags') {
            include(__DIR__ . "/../lib/assets/static/images/flags/{$filename}");
        } elseif (strpos($filename, '.css')) {
            include(__DIR__ . "/../lib/assets/static/{$filename}");
        } elseif (strpos($filename, '.js')) {
            include(__DIR__ . "/../lib/assets/static/{$filename}");
        } else {
            include(__DIR__ . "/../lib/assets/static/{$filename}");
        }
    }

    /**
     * Executes the classes interface.
     *
     * @param string $filename
     */
    public function runClasses($filename)
    {
        include(Ini::get(Ini::IWORX_DIR, 'base') . "/plugins/imunifyav/lib/classes/{$filename}");
    }

    /**
     * Integrate into the nodeworx menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateNodeworxMenu(IWorxMenuManager $MenuMan)
    {
        $this->_addNodeworxImunifyAvMenu($MenuMan);
    }

    /**
     * Integrate into the siteworx menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    public function updateSiteworxMenu(IWorxMenuManager $MenuMan)
    {
        $this->_addSiteworxImunifyAvMenu($MenuMan);
    }

    /**
     * Add siteworx imunify menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addSiteworxImunifyAvMenu(IWorxMenuManager $MenuMan)
    {
        $name = $this->_getApplicationName();
        $menu_data = array(
          'text'  => $name,
          'url'   => '/siteworx/imunifyav',
          'class' => 'mdi-cloud-outline',
        );

        $MenuMan->addMenuItemAfter(
            'iw-menu-pref-group',
            'imunifyav',
            $menu_data
        );
    }

    /**
     * Add nodeworx imunify menu.
     *
     * @param IWorxMenuManager $MenuMan
     */
    private function _addNodeworxImunifyAvMenu(IWorxMenuManager $MenuMan)
    {
        $name = $this->_getApplicationName();
        $menu_data = array(
          'text'                  => $name,
          'url'                   => '/nodeworx/imunifyav',
          'class'                 => 'mdi-cloud-outline',
          'disabled_for_reseller' => '1'
        );

        $MenuMan->addMenuItemAfter(
            'iw-menu-server',
            'imunifyav',
            $menu_data
        );
    }

    /**
     * Returns the name of Imunify application running.
     *
     * @return string
     */
    private function _getApplicationName()
    {
        $application = Ini::get(Ini::IMUNIFY, 'application');
        if ($application === self::AV) {
            return 'ImunifyAV';
        } else {
            return 'Imunify360';
        }
    }
}
